"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TRenderEngine = void 0;

var _collapse = require("./flow/collapse");

var _hoist = require("./flow/hoist");

var _translate = require("./flow/translate");

var _htmlparser = require("htmlparser2");

var _omit = _interopRequireDefault(require("ramda/src/omit"));

var _cssProcessor = require("@native-html/css-processor");

var _TStylesMerger = require("./styles/TStylesMerger");

var _defaults = require("./styles/defaults");

var _TStyles = require("./styles/TStyles");

var _HTMLModelRegistry = _interopRequireDefault(require("./model/HTMLModelRegistry"));

var _alterDOMNodes = _interopRequireDefault(require("./dom/alterDOMNodes"));

var _domUtils = require("./dom/dom-utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createStylesConfig(options) {
  var _options$stylesConfig, _options$stylesConfig2;

  const enableUserAgentStyles = typeof (options === null || options === void 0 ? void 0 : (_options$stylesConfig = options.stylesConfig) === null || _options$stylesConfig === void 0 ? void 0 : _options$stylesConfig.enableUserAgentStyles) === 'boolean' ? options.stylesConfig.enableUserAgentStyles : _defaults.defaultStylesConfig.enableUserAgentStyles;
  const baseStyle = { ...(enableUserAgentStyles ? _defaults.defaultStylesConfig.baseStyle : (0, _omit.default)(['fontSize'], _defaults.defaultStylesConfig.baseStyle)),
    ...(options === null || options === void 0 ? void 0 : (_options$stylesConfig2 = options.stylesConfig) === null || _options$stylesConfig2 === void 0 ? void 0 : _options$stylesConfig2.baseStyle)
  };
  return { ..._defaults.defaultStylesConfig,
    ...(options === null || options === void 0 ? void 0 : options.stylesConfig),
    baseStyle
  };
}

class TRenderEngine {
  constructor(options) {
    var _options$dangerouslyD, _options$dangerouslyD2, _options$cssProcessor, _stylesConfig$baseSty;

    const stylesConfig = createStylesConfig(options);
    this.hoistingEnabled = !((_options$dangerouslyD = options === null || options === void 0 ? void 0 : options.dangerouslyDisableHoisting) !== null && _options$dangerouslyD !== void 0 ? _options$dangerouslyD : false);
    this.whitespaceCollapsingEnabled = !((_options$dangerouslyD2 = options === null || options === void 0 ? void 0 : options.dangerouslyDisableWhitespaceCollapsing) !== null && _options$dangerouslyD2 !== void 0 ? _options$dangerouslyD2 : false);
    const modelRegistry = new _HTMLModelRegistry.default(options === null || options === void 0 ? void 0 : options.customizeHTMLModels);
    const userSelectedFontSize = (options === null || options === void 0 ? void 0 : (_options$cssProcessor = options.cssProcessorConfig) === null || _options$cssProcessor === void 0 ? void 0 : _options$cssProcessor.rootFontSize) || ((_stylesConfig$baseSty = stylesConfig.baseStyle) === null || _stylesConfig$baseSty === void 0 ? void 0 : _stylesConfig$baseSty.fontSize); // TODO log a warning when type is string

    const stylesMerger = new _TStylesMerger.TStylesMerger(stylesConfig, modelRegistry, { ..._cssProcessor.defaultCSSProcessorConfig,
      ...(options === null || options === void 0 ? void 0 : options.cssProcessorConfig),
      rootFontSize: typeof userSelectedFontSize === 'number' ? userSelectedFontSize : 14
    });
    this.alterDOMParams = options === null || options === void 0 ? void 0 : options.alterDOMParams;
    this.htmlParserOptions = {
      decodeEntities: true,
      lowerCaseTags: true,
      ...(options === null || options === void 0 ? void 0 : options.htmlParserOptions)
    };
    this.dataFlowParams = {
      stylesMerger,
      modelRegistry,
      baseStyles: new _TStyles.TStyles(stylesMerger.compileStyleDeclaration(stylesConfig.baseStyle)),
      removeLineBreaksAroundEastAsianDiscardSet: (options === null || options === void 0 ? void 0 : options.removeLineBreaksAroundEastAsianDiscardSet) || false
    };
  }

  normalizeDocument(document) {
    let body;
    let head;

    for (const child of document.children) {
      if (body && head) {
        break;
      }

      if ((0, _domUtils.isDOMElement)(child) && child.tagName === 'body') {
        body = child;
      }

      if ((0, _domUtils.isDOMElement)(child) && child.tagName === 'head') {
        head = child;
      }
    } //@ts-ignore


    if (!body && !head) {
      body = new _domUtils.DOMElement('body', {});
      body.childNodes = document.children;
      document.children.forEach(c => {
        c.parent = body;
        c.parentNode = body;
      });
      body.parent = document;
      body.parentNode = document;
      document.childNodes = [body];
    }

    return document;
  }

  applyDOMTampering(document) {
    var _this$alterDOMParams, _this$alterDOMParams2, _this$alterDOMParams3, _this$alterDOMParams4;

    if ((_this$alterDOMParams = this.alterDOMParams) !== null && _this$alterDOMParams !== void 0 && _this$alterDOMParams.ignoreDOMNode || (_this$alterDOMParams2 = this.alterDOMParams) !== null && _this$alterDOMParams2 !== void 0 && _this$alterDOMParams2.alterDOMChildren || (_this$alterDOMParams3 = this.alterDOMParams) !== null && _this$alterDOMParams3 !== void 0 && _this$alterDOMParams3.alterDOMData || (_this$alterDOMParams4 = this.alterDOMParams) !== null && _this$alterDOMParams4 !== void 0 && _this$alterDOMParams4.alterDOMElement) {
      document.childNodes = (0, _alterDOMNodes.default)(document.childNodes, this.alterDOMParams);
    }

    return document;
  }

  parseDocument(html) {
    let document = (0, _htmlparser.parseDocument)(html, this.htmlParserOptions);

    for (const child of document.children) {
      if ((0, _domUtils.isDOMElement)(child) && child.tagName === 'html') {
        document = child;
        break;
      }
    }

    return this.normalizeDocument(document);
  }

  buildTTreeFromDoc(document) {
    const tamperedDoc = this.applyDOMTampering(document);
    const tdoc = (0, _translate.translateDocument)(tamperedDoc, this.dataFlowParams);
    const hoistedTDoc = this.hoistingEnabled ? (0, _hoist.hoist)(tdoc) : tdoc;
    const collapsedTDoc = this.whitespaceCollapsingEnabled ? (0, _collapse.collapse)(hoistedTDoc, this.dataFlowParams) : tdoc;
    return collapsedTDoc;
  }

  buildTTree(html) {
    return this.buildTTreeFromDoc(this.parseDocument(html));
  }

}

exports.TRenderEngine = TRenderEngine;
//# sourceMappingURL=TRenderEngine.js.map