"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = alterDOMNodes;

var _map = _interopRequireDefault(require("ramda/src/map"));

var _identity = _interopRequireDefault(require("ramda/src/identity"));

var _always = _interopRequireDefault(require("ramda/src/always"));

var _pipe = _interopRequireDefault(require("ramda/src/pipe"));

var _reject = _interopRequireDefault(require("ramda/src/reject"));

var _domUtils = require("./dom-utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function alterDOMNodes(nodes, params) {
  const {
    ignoreDOMNode = (0, _always.default)(false),
    alterDOMElement = _identity.default,
    alterDOMChildren = _identity.default,
    alterDOMData = node => node.data
  } = params;
  const alterNodes = (0, _pipe.default)((0, _reject.default)(ignoreDOMNode), (0, _map.default)(function applyParams(node) {
    let nextNode = node;

    if ((0, _domUtils.isDOMElement)(node)) {
      const elementNode = alterDOMElement(node) || nextNode;
      const alteredChildren = alterDOMChildren(elementNode);
      elementNode.children = Array.isArray(alteredChildren) ? alteredChildren : elementNode.children;
      elementNode.children = alterNodes(elementNode.children);
      nextNode = elementNode;
    } else if ((0, _domUtils.isDOMText)(node)) {
      const textNode = node;
      const alteredData = alterDOMData(textNode);
      textNode.data = typeof alteredData === 'string' ? alteredData : textNode.data;
    }

    return nextNode;
  }));
  return alterNodes(nodes);
}
//# sourceMappingURL=alterDOMNodes.js.map