"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = serializeTNode;

function printTableStyles(styles) {
  const allStyles = { ...styles.nativeTextFlow,
    ...styles.nativeBlockFlow,
    ...styles.webTextFlow,
    ...styles.nativeTextRet,
    ...styles.nativeBlockRet
  };
  const entries = Object.entries(allStyles);
  return entries.reduce((prev, [name, val]) => {
    return "".concat(prev).concat(prev ? ',' : '', " ").concat(name, ": ").concat(JSON.stringify(val));
  }, '');
}

function tnodePropertiesString(tnode) {
  var _tnode$classes;

  const tagPrint = tnode.tagName ? "tagName=\"".concat(tnode.tagName, "\"") : 'anonymous';
  const unregisteredPrint = tnode.isUnregistered ? 'unregistered' : null;
  const indexPrint = "nodeIndex={".concat(tnode.nodeIndex, "}");
  const idPrint = tnode.id ? "id=".concat(tnode.id) : null;
  const classesPrint = (_tnode$classes = tnode.classes) !== null && _tnode$classes !== void 0 && _tnode$classes.length ? "classes={[".concat(tnode.classes.join(', '), "]}") : null;
  const dataPrint = tnode.type === 'text' ? "data=".concat(JSON.stringify(tnode.data)) : null;
  const anchorPrint = typeof tnode.attributes.href === 'string' ? "href=".concat(JSON.stringify(tnode.attributes.href)) : null;
  const srcPrint = typeof tnode.attributes.src === 'string' ? "src=".concat(JSON.stringify(tnode.attributes.src)) : null;
  const pstyles = printTableStyles(tnode.styles);
  const stylesPrint = pstyles ? "styles={{".concat(pstyles, " }}") : null;
  const detailsPrint = [tagPrint, unregisteredPrint, indexPrint, idPrint, classesPrint, dataPrint, anchorPrint, srcPrint, stylesPrint].filter(p => p !== null).join(' ');
  return "<".concat(tnode.displayName, " ").concat(detailsPrint);
}

function serializeTNode(tnode, params = {}) {
  const {
    parentLeftPrefix = '',
    isChild = false,
    isLast = false
  } = params;
  const prefix = isChild ? '  ' : '';
  const totalPrefixLeft = parentLeftPrefix + prefix;
  const childrenPrint = tnode.children.map((c, i) => serializeTNode(c, {
    parentLeftPrefix: parentLeftPrefix + ' '.padStart(prefix.length, ' '),
    isChild: true,
    isLast: i === tnode.children.length - 1
  })).join('');
  return "".concat(totalPrefixLeft).concat(tnodePropertiesString(tnode)).concat(childrenPrint ? ">\n".concat(childrenPrint, "\n").concat(totalPrefixLeft, "</").concat(tnode.displayName, ">") : ' />').concat(isLast ? '' : '\n');
}
//# sourceMappingURL=tnodeToString.js.map