import { ParserOptions as HTMLParserOptions } from 'htmlparser2';
import { CSSProcessorConfig } from '@native-html/css-processor';
import { StylesConfig } from './styles/types';
import { HTMLModelRecord, TagName } from './model/model-types';
import { DefaultHTMLElementModels } from './model/defaultHTMLElementModels';
import { AlterDOMParams } from './dom/alterDOMNodes';
import { DOMDocument, DOMElement } from './dom/dom-utils';
import { TDocument } from './tree/tree-types';
export interface TRenderEngineOptions<E extends string = never> {
    /**
     * Customization for CSS inline processing.
     */
    readonly cssProcessorConfig?: Partial<CSSProcessorConfig>;
    /**
     * Options for htmlparser2 library parser.
     */
    readonly htmlParserOptions?: Readonly<HTMLParserOptions>;
    /**
     * Various configuration for styling.
     */
    readonly stylesConfig?: StylesConfig;
    /**
     * Customize supported tags in the engine.
     *
     * @remarks If you need to add new tags, always use lowercase names.
     */
    readonly customizeHTMLModels?: (defaultHTMLElementModels: DefaultHTMLElementModels) => HTMLModelRecord<TagName | E>;
    /**
     * Remove line breaks around special east-asian characters such as defined here:
     * https://www.w3.org/TR/2020/WD-css-text-3-20200429/#line-break-transform
     *
     * @defaultValue false
     */
    readonly removeLineBreaksAroundEastAsianDiscardSet?: boolean;
    /**
     * Alter the DOM tree prior to pre-rendering.
     */
    readonly alterDOMParams?: AlterDOMParams;
    /**
     * Disable hoisting. Note that your layout might break!
     */
    readonly dangerouslyDisableHoisting?: boolean;
    /**
     * Disable whitespace collapsing. Especially useful if your html is
     * being pre-processed server-side with a minifier.
     */
    readonly dangerouslyDisableWhitespaceCollapsing?: boolean;
}
export declare class TRenderEngine {
    private htmlParserOptions;
    private dataFlowParams;
    private alterDOMParams?;
    private hoistingEnabled;
    private whitespaceCollapsingEnabled;
    constructor(options?: TRenderEngineOptions);
    private normalizeDocument;
    private applyDOMTampering;
    parseDocument(html: string): DOMDocument;
    buildTTreeFromDoc(document: DOMDocument | DOMElement): TDocument;
    buildTTree(html: string): TDocument;
}
