import { MixedStyleDeclaration } from '@native-html/css-processor';
import HTMLContentModel from './HTMLContentModel';
import { CustomElementModel, ElementCategory, NativeElementModel, TagName } from './model-types';
export interface HTMLElementModelProperties<T extends string, M extends HTMLContentModel> {
    readonly tagName: T;
    readonly contentModel: M;
    readonly isOpaque: boolean;
    readonly isVoid: boolean;
    readonly mixedUAStyles?: MixedStyleDeclaration;
    /**
     * Examples:
     *
     * - "width" and "height" attributes for &lt;img&gt; tag;
     * - "cite" attribute for &lt;blockquote&gt; tag.
     */
    readonly getUADerivedStyleFromAttributes?: (attributes: Record<string, string>) => MixedStyleDeclaration | null;
}
/**
 * An object defining engine internals for tags, such as default styles
 * (UAStyles), content model (how this tag is treated during hoisting)... etc.
 */
export default class HTMLElementModel<T extends string, M extends HTMLContentModel> implements HTMLElementModelProperties<T, M> {
    readonly tagName: T;
    readonly contentModel: M;
    readonly isOpaque: boolean;
    readonly isVoid: boolean;
    readonly mixedUAStyles?: MixedStyleDeclaration;
    readonly getUADerivedStyleFromAttributes: NativeElementModel['getUADerivedStyleFromAttributes'];
    private constructor();
    static fromCustomModel<T extends string, M extends HTMLContentModel>({ contentModel, tagName, isOpaque, isVoid, ...optionalFields }: CustomElementModel<Exclude<T, TagName>, M>): HTMLElementModel<Exclude<T, TagName>, M>;
    static fromNativeModel<T extends TagName, E extends ElementCategory>({ tagName, category, isOpaque, mixedUAStyles, isVoid, getUADerivedStyleFromAttributes }: NativeElementModel<T, E>): HTMLElementModel<T, E extends "anchor" | "edits" ? HTMLContentModel.mixed : E extends "tabular" | "sectioning" | "grouping" ? HTMLContentModel.block : E extends "textual" ? HTMLContentModel.textual : HTMLContentModel.none>;
    isTranslatableBlock(): boolean;
    isTranslatableTextual(): boolean;
    extend<N extends HTMLContentModel>(props: Partial<HTMLElementModelProperties<T, N>>): HTMLElementModel<T, N>;
    getUADerivedCSSProcessedPropsFromAttributes(attributes: Record<string, string>): MixedStyleDeclaration | null;
}
