import { MixedStyleDeclaration } from '@native-html/css-processor';
import HTMLContentModel from './HTMLContentModel';
import HTMLElementModel from './HTMLElementModel';
export declare type ElementCategory = 'anchor' | 'textual' | 'tabular' | 'edits' | 'embedded' | 'sectioning' | 'grouping' | 'interactive' | 'untranslatable';
export declare type TagName = AnchorTagName | TextLevelTagNames | EditsTagNames | EmbeddedTagNames | TabularTagNames | GroupingTagNames | SectioningTagNames | InteractiveTagNames | UntranslatableTagNames;
export declare type AnchorTagName = 'a';
export declare type TabularTagNames = 'table' | 'tbody' | 'thead' | 'tfoot' | 'tr' | 'td' | 'th';
export declare type GroupingTagNames = 'p' | 'hr' | 'pre' | 'blockquote' | 'ol' | 'ul' | 'dir' | 'menu' | 'li' | 'dl' | 'dt' | 'dd' | 'figure' | 'figcaption' | 'main' | 'div' | 'xmp' | 'listing' | 'plaintext';
export declare type AttribTagNames = 'accesskey' | 'datalist' | 'source' | 'track' | 'caption' | 'colgroup' | 'col' | 'option' | 'optgroup' | 'param';
export declare type MetadataTagNames = 'head' | 'title' | 'base' | 'link' | 'meta';
export declare type InteractiveTagNames = 'form' | 'label' | 'input' | 'button' | 'select' | 'progress' | 'meter' | 'fieldset' | 'legend' | 'textarea' | 'output' | 'details' | 'summary' | 'dialog';
export declare type UnsupportedTagNames = 'area' | 'map';
export declare type UntranslatableTagNames = AttribTagNames | UnsupportedTagNames | MetadataTagNames;
export declare type EmbeddedTagNames = 'audio' | 'canvas' | 'embed' | 'iframe' | 'img' | 'math' | 'object' | 'picture' | 'svg' | 'video';
export declare type EditsTagNames = 'ins' | 'del';
export declare type SectioningTagNames = 'body' | 'article' | 'section' | 'nav' | 'aside' | 'h1' | 'h2' | 'h3' | 'h4' | 'h5' | 'h6' | 'hgroup' | 'header' | 'footer' | 'address';
export declare type TextLevelTagNames = 'em' | 'strong' | 'strike' | 'small' | 'big' | 's' | 'cite' | 'q' | 'dfn' | 'abbr' | 'acronym' | 'ruby' | 'rt' | 'rp' | 'data' | 'time' | 'tt' | 'code' | 'var' | 'samp' | 'kbd' | 'sup' | 'sub' | 'i' | 'b' | 'u' | 'mark' | 'bdi' | 'bdo' | 'span' | 'br' | 'wbr';
export declare type HTMLModelRecord<T extends string = TagName, M extends HTMLContentModel = HTMLContentModel> = {
    [k in T]: HTMLElementModel<k, M>;
};
export interface ElementModelBase<T extends string> {
    /**
     * The tag name associated with this model.
     */
    tagName: T;
    /**
     * An opaque element translated TNode will have a `domNode` property.
     */
    isOpaque?: boolean;
    /**
     * Equivalent of "user-agent" styles. The default styles for the element.
     *
     * @remarks These styles will get merged over by `tagsStyles`.
     */
    mixedUAStyles?: MixedStyleDeclaration;
    /**
     * Conditional "user-agent" styles.
     *
     * @remarks For example, &lt;a&gt; tags will have underline decoration and be
     * colored blue only when `href` is defined.
     */
    getUADerivedStyleFromAttributes?: (attributes: Record<string, string>) => MixedStyleDeclaration | null;
}
/**
 * An object to specify custom tags.
 */
export interface CustomElementModel<T extends string, M extends HTMLContentModel> extends ElementModelBase<T> {
    tagName: Exclude<T, TagName>;
    contentModel: M;
    isVoid?: boolean;
}
/**
 * An object to specify tags parts of the HTML4 and HTML5 standards.
 */
export interface NativeElementModel<T extends string = TagName, C = ElementCategory> extends ElementModelBase<T> {
    /**
     * Void elements such as specified in HTML4:
     *
     * - Void elements cannot have children.
     * - TText-translated void elements will be preserved even though they don't have children.
     */
    isVoid?: boolean;
    /**
     * The category of this element as per HTML5 standard.
     */
    category: C;
}
